 /*

  This was a part of the LiteStep Shell Source code.

  Copyright (C) 1997-98 The LiteStep Development Team

  Now it's an heavily modified source and is no more part
  of the LS Shell Source code.
  The hacks are Copyright (C) 1999 _TAG_ (Togna Alessandro Giorgio)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <windows.h>
#include <stdio.h>
#include <shlobj.h>
#include <tchar.h>
#include <process.h>
#include "popup.h"

const char rcsRevision[] = "$Revision: 0.1 $"; // Our Version 
const char rcsId[] = "$Id: popup.cpp,v 0.1 1999/03/01 14:21:26 _TAG_ Exp $"; // The Full RCS ID.
char popupFontFace[256];
char popupMenuName[256];
const char szAppName[] = "LPopupMenu"; // Our window class, etc

int popupSubHeight = 20;
int popupMinWidth = 100;
int popupIconSize = 20;
int titleHeight = 20;
int popupFixedWidth = 0;
int popupMenuDelay = 0;
int popupFontHeight;
int popupTextOffset;
int popupTop;

menuItem *previousItem = NULL;	
menuItem *timerPointer = NULL;

HFONT popupFont;
HICON popupDefaultIcon;
COLORREF titleColor, selColor, normColor;
COLORREF titleBack,selBack, normBack;
WNDCLASS wc;

Popup popup; //popup class struct
fileItem *popupFile, *treeItem;  //rc file info holder

HINSTANCE appInstance;
HWND itemCaller = NULL;
HWND parentWin;
HWND hMainWnd = NULL;

BOOL wentRight[255];
BOOL noPopupBevel;
BOOL noPopupPix = FALSE;
BOOL transBlt=TRUE;
BOOL popupFolderIcon=TRUE;
BOOL showPopupIcons = FALSE;
BOOL isFixedWidth = FALSE;
BOOL shrinkPopupBar = FALSE;

BitMP backBMP;
BitMP titleBMP;
BitMP selectBMP;
BitMP bottomBMP;

int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath);
void initializePopupMenu();
void displayPopup(HWND hwnd, int x, int y);
void paintPopupBevel(HWND hwnd, menuItem* item, int totalHeight, int width);
void executeItem(menuItem* pItem);
void keyedMenu(HWND Menu, menuItem *showItem);
void buildTaskList();
void setPopupValues();
menuItem *GetItem(menuItem *root, int num);
HRGN GetRegion(menuItem *item);

////////////////////////////////////////////////////////////////////////////////
//BEGIN CLASS DEFINITION 
////////////////////////////////////////////////////////////////////////////////

menuItem::~menuItem() {
   	if(next != NULL)
   		delete next;
	if (hIcon != NULL)
		DeleteObject(hIcon);
}

int Popup::returnMenuWidth()
{
    return(menuWidth);
}


fileItem::fileItem()
{
    strcpy(itemName, "\0");
    strcpy(bmpName, " ");
    strcpy(popupCommand, " ");
    next = NULL;
    itemHeight = popupSubHeight;
}

fileItem::~fileItem()
{
    delete next;
}

BOOL Popup::delAllMenuItems()
{
	if(root != NULL)
		delete root;
	root = NULL;
	current = NULL;
	return(TRUE);
}

BOOL Popup::newMenuItem()
{
    menuItem *temp = root;
	menuItem *tprv = NULL;
    
    while(1) //finds the last node in the list
    {
    	if(temp == NULL)
    		break;
		else if(temp->next == NULL) {
			tprv = temp;
    		break;
		} else {
			tprv = temp;
			temp = temp->next;
		}
    }
    
    if(temp == NULL)
    {
    	root = new menuItem;
    	current = root;
    }
    else 
    {
    	temp->next = new menuItem;
    	current = temp->next;
		current->prev = tprv;
    }
    
    return(TRUE);
}

// window procedure for our window
LRESULT CALLBACK Popup::wndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int minWidth, totHeight;
	POINT pt;
    RECT r;
    menuItem *item;
	Popup *pop;

   	GetCursorPos(&pt);
   	GetWindowRect(hwnd, &r);
   	pt.x -= r.left;
   	pt.y -= r.top;
	
	pop = &popup;
	minWidth = pop->menuWidth;
	totHeight = pop->totalHeight+titleHeight;
	
    switch (message) {
	case LM_GETREVID: {
		char *buf = (char *) lParam;

		if (wParam == 0) {
			strcpy(buf, "lpopup.dll: ");
			strcat(buf, &rcsRevision[11]);
			buf[strlen(buf)-1] = '\0';
		} else if (wParam == 1) {
			strcpy(buf, &rcsId[1]);
			buf[strlen(buf)-1] = '\0';
		} else {
			strcpy(buf, "");
		}
		return strlen(buf);
	}

	case LM_LPOPUP:
		GetCursorPos(&pt);
		displayPopup(hwnd, pt.x, pt.y);
		SetForegroundWindow(hwnd);
		return 0;
	case WM_PAINT:
		pop->paintPopup();
		if (pop->pinned) {
			DrawFrameControl(GetDC(hwnd), &pop->close, DFC_CAPTION, DFCS_CAPTIONCLOSE|DFCS_FLAT);
		}
		break;

	case WM_ERASEBKGND:
		return 0;

    case WM_TIMER:
    	KillTimer(hwnd, 1);
    	if(timerPointer == NULL)
    		break;
    case WM_MOUSEMOVE: 
	{
	    int y = HIWORD(lParam); 
		int prevHeight;
		int itemNo = y / popupSubHeight;
		
		item = GetItem(pop->root, itemNo);
		if (previousItem == item)
			break;
		if(y < titleHeight || y>=totHeight) {
			// Do nothing
		} else {
			if (previousItem != NULL) {
				prevHeight = (previousItem->itemNo*popupSubHeight);
				if(y < prevHeight || y > prevHeight)
    			{
					timerPointer = NULL;
					if(pop->isSelected != NULL) {
		    			pop->selectItem(FALSE);
					}
    			}
			}
			pop->isSelected = item;
			pop->selectItem(TRUE);
			previousItem = item;
   		}
    	break;
	}
    case WM_NCHITTEST:
    	if(pt.y < titleHeight ) {
			if (pop->pinned) {
				if (PtInRect(&pop->close, pt)) {
					return (HTCLOSE);
				} else
					return (HTCAPTION);
			} else
   				return (HTCAPTION);
	   	}
   		else break;

    case WM_NCLBUTTONDBLCLK:
		if (!pop->pinned) {
			pop->pinned = TRUE;
			if (pop != &popup) {
				pop = &popup;
				ShowWindow(pop->hwnd, SW_HIDE );
			}
			InvalidateRect(hwnd, NULL, FALSE);
		} else {
			pop->pinned = FALSE;
			InvalidateRect(hwnd, NULL, FALSE);
		}
		break;

	case WM_NCLBUTTONDOWN:
		if (pop->pinned) {
			if (PtInRect(&pop->close, pt)) {
				pop->pinned = FALSE;
				ShowWindow(pop->hwnd, SW_HIDE);
			}
		}
		break;
	case LM_HIDEPOPUP:
		pop->closeAll();
		previousItem = NULL;
		return 0;
    case WM_ACTIVATE:
		{
			HWND hwndGetFocus;
		    WORD fActive;
			fActive = LOWORD(wParam);

    		if(fActive == WA_INACTIVE) {
	    		hwndGetFocus = (HWND)lParam; /* handle of window receiving focus */
				if (hwndGetFocus == NULL || hwndGetFocus!=hwnd) {
					pop->closeAll();
					previousItem = NULL;
				}
			}
			return 0;
	    }
    	break;

    case WM_LBUTTONUP:
		if (pt.y>titleHeight && pt.y<=totHeight) {
	  		if(pop->isSelected != NULL)
   			{
				item = pop->isSelected;
				executeItem(item);
				pop = &popup; // Put in code later to verify open popup
				if (!pop->pinned) {
					previousItem = NULL;
					pop->isSelected = NULL;
   					ShowWindow(pop->hwnd, SW_HIDE );
				}
   				break;
			}
		}
    	break;
	case WM_KEYDOWN: 
		switch (wParam) {
		case VK_ESCAPE:
			pop->closeAll();
			previousItem = NULL;
			break;

		case VK_DOWN: 
			if (pop->isSelected == NULL || pop->isSelected == pop->current) {
				keyedMenu(pop->hwnd, pop->root);
			} else {
				keyedMenu(pop->hwnd, pop->isSelected->next);
			}
			break;

		case VK_UP: 
			if (pop->isSelected == NULL || pop->isSelected == pop->root) {
				keyedMenu(pop->hwnd, pop->current);
			} else {
				keyedMenu(pop->hwnd, pop->isSelected->prev);
			}
			break;
		case VK_RETURN: //Sub folder
			if (pop->isSelected != NULL) {
	    		char temp[256];
				executeItem(pop->isSelected);
			} 
			break;
		default:
			if (isalnum(wParam)) {
				item = pop->keyItem(wParam);
				if (item != NULL) 
					keyedMenu(pop->hwnd, item);
			}
		}
		break;
	}
    return DefWindowProc(hwnd, message, wParam, lParam);
}

void Popup::selectItem(BOOL selected)
{
	HWND hwnd;
	menuItem *item;
    HDC DC, memDC;
    PAINTSTRUCT ps;
    HFONT oldFont;
	HBRUSH hBrush;
    RECT r;
	int totalHeight, bWidth, bHeight;
	static menuItem *paintedLastTime = NULL;
	static BOOL bWasSelected = FALSE;

	item = this->isSelected;
	totalHeight = item->itemNo*popupSubHeight;
	hwnd = this->hwnd;
	// Test if it is the same menu item thet we painted last time
	// is so return, this will reduce flicker.
	if(paintedLastTime == item && bWasSelected == selected && previousItem != NULL)
		return;
	else{
		bWasSelected = selected;
		paintedLastTime = item;
	}
	int minWidth;
	minWidth = this->menuWidth;

   	DC = BeginPaint(hwnd,&ps);
   	memDC = CreateCompatibleDC(DC);
   	DC = GetDC(hwnd);
   	oldFont = (HFONT) SelectObject(DC, popupFont);

    if(selected) {
		if (!noPopupPix) {
		   	SelectObject(memDC, selectBMP.bitmap);
//   			BitBlt(DC,0,totalHeight, selectBMP.x, popupSubHeight, memDC, 0, 0,SRCCOPY);
			if (shrinkPopupBar) {
				StretchBlt(DC,0,totalHeight, menuWidth, popupSubHeight, memDC, 0, 0,selectBMP.x, selectBMP.y, SRCCOPY);
			} else {
				for(int x=0; x<menuWidth; x+=selectBMP.x)
					for(int y=totalHeight+1; y<(totalHeight+popupSubHeight); y+=selectBMP.y) {
						bWidth = x+selectBMP.x<menuWidth?selectBMP.x:x<menuWidth?menuWidth-x:menuWidth;
						bHeight = y+selectBMP.y<(totalHeight+popupSubHeight)?selectBMP.y:y<popupSubHeight?popupSubHeight-y:popupSubHeight;
						BitBlt(DC, x, y, bWidth, bHeight, memDC, 0, 0, SRCCOPY);
					}
			}
		} else {
			hBrush = CreateSolidBrush(selBack);
			SetRect(&r, 0, totalHeight,minWidth, totalHeight+titleHeight);
			FillRect(DC, &r, hBrush);
			DeleteObject(hBrush);
		}
    	SetTextColor(DC,  selColor);
	   	SetBkMode(DC, TRANSPARENT);
    	r.right = selectBMP.x-popupTextOffset;
    } else {
		if (!noPopupPix) {
		   	SelectObject(memDC, backBMP.bitmap);
			if (shrinkPopupBar) {
				StretchBlt(DC,0,totalHeight, menuWidth, popupSubHeight, memDC, 0, 0,backBMP.x, backBMP.y, SRCCOPY);
			} else {
				for(int x=0; x<menuWidth; x+=backBMP.x)
					for(int y=totalHeight+1; y<(totalHeight+popupSubHeight); y+=backBMP.y) {
						bWidth = x+backBMP.x<menuWidth?backBMP.x:x<menuWidth?menuWidth-x:menuWidth;
						bHeight = y+backBMP.y<(totalHeight+popupSubHeight)?backBMP.y:y<popupSubHeight?popupSubHeight-y:popupSubHeight;
						BitBlt(DC, x, y, bWidth, bHeight, memDC, 0, 0, SRCCOPY);
					}
			}
		} else {
			hBrush = CreateSolidBrush(normBack);
			SetRect(&r, 0, totalHeight,minWidth, totalHeight+titleHeight);
			FillRect(DC, &r, hBrush);
			DeleteObject(hBrush);
		}
    	SetTextColor(DC,  normColor);
	   	SetBkMode(DC, TRANSPARENT);
    	r.right = backBMP.x-popupTextOffset;
    }
	//Draw Icon
	if (showPopupIcons) {
		if (item->hIcon != NULL)
			DrawIconEx(DC, popupTextOffset, totalHeight+popupTop, item->hIcon, popupIconSize, popupIconSize, 0, NULL, DI_NORMAL); // DI_DEFAULTSIZE
		r.left = popupTextOffset+popupIconSize+4;
	} else
		r.left = popupTextOffset;
   	r.top= totalHeight;
	r.bottom = totalHeight + popupSubHeight;
   	DrawText(DC, item->name, lstrlen(item->name), &r, DT_VCENTER|DT_SINGLELINE|DT_NOCLIP);

   	if (!noPopupBevel) {
		paintPopupBevel(hwnd, item, totalHeight, minWidth);
    }
   	SelectObject(DC, oldFont);
	DeleteObject(hBrush);
   	ReleaseDC(hwnd, DC);
   	EndPaint(hwnd,&ps);	
   	DeleteDC(memDC);
}

void Popup::paintPopup() {
    HFONT oldFont;
    RECT r;
    HDC DC, memDC;
    PAINTSTRUCT ps;
	HBRUSH hBrush;
	char *name;
	int menuWidth, bWidth, bHeight;
    
	buildTaskList();
	name = popupMenuName;
    menuItem *iTemp = this->root;
	menuWidth = this->menuWidth;

    DC = BeginPaint(this->hwnd,&ps);
    memDC = CreateCompatibleDC(DC);
    oldFont = (HFONT) SelectObject(DC, popupFont);
	if (!noPopupPix) {
	    //blits out title info
		SelectObject(memDC, titleBMP.bitmap);
		if (shrinkPopupBar) {
			StretchBlt(DC,0,0, menuWidth, titleHeight, memDC, 0, 0,titleBMP.x, titleBMP.y, SRCCOPY);
		} else {
			for(int x=0; x<menuWidth; x+=titleBMP.x)
				for(int y=0; y<titleHeight; y+=titleBMP.y) {
					bWidth = x+titleBMP.x<menuWidth?titleBMP.x:x<menuWidth?menuWidth-x:menuWidth;
					bHeight = y+titleBMP.y<titleHeight?titleBMP.y:y<titleHeight?titleHeight-y:titleHeight;
					BitBlt(DC, x, y, bWidth, bHeight, memDC, 0, 0, SRCCOPY);
				}
		}
	} else {
		hBrush = CreateSolidBrush(titleBack);
		SetRect(&r, 0, 0, menuWidth, titleHeight);
		FillRect(DC, &r, hBrush);
		DeleteObject(hBrush);
	}
    SetBkMode(DC, TRANSPARENT);

    r.left = popupTextOffset;
    r.right = titleBMP.x-popupTextOffset;
    r.top= 0;
    r.bottom = titleHeight;
    SetTextColor(DC,  titleColor);
	DrawText(DC, name, lstrlen(name), &r, DT_VCENTER|DT_SINGLELINE|DT_NOCLIP);
    SelectObject(DC, oldFont);
    ReleaseDC(this->hwnd, DC);

    int totalHeight = titleHeight;
    //blits out each menu item, in order that they where entered into the RC
   	DC = GetDC(this->hwnd);
   	oldFont = (HFONT) SelectObject(DC, popupFont);
   	SelectObject(memDC, backBMP.bitmap);
    while(iTemp != NULL)
    {
		if (!noPopupPix) {
			if (shrinkPopupBar) {
				StretchBlt(DC,0,totalHeight, menuWidth, popupSubHeight, memDC, 0, 0,backBMP.x, backBMP.y, SRCCOPY);
			} else {
				for(int x=0; x<menuWidth; x+=backBMP.x)
					for(int y=totalHeight+1; y<(totalHeight+popupSubHeight); y+=backBMP.y) {
						bWidth = x+backBMP.x<menuWidth?backBMP.x:x<menuWidth?menuWidth-x:menuWidth;
						bHeight = y+backBMP.y<(totalHeight+popupSubHeight)?backBMP.y:y<popupSubHeight?popupSubHeight-y:popupSubHeight;
						BitBlt(DC, x, y, bWidth, bHeight, memDC, 0, 0, SRCCOPY);
					}
			}
		} else {
			hBrush = CreateSolidBrush(normBack);
			SetRect(&r, 0, totalHeight, menuWidth, totalHeight+titleHeight);
			FillRect(DC, &r, hBrush);
			DeleteObject(hBrush);
		}
		SetTextColor(DC,  normColor);
    	SetBkMode(DC, TRANSPARENT);
		//Draw Icon
		if (showPopupIcons) {
			if (iTemp->hIcon != NULL)
				DrawIconEx(DC, popupTextOffset, totalHeight+popupTop, iTemp->hIcon, popupIconSize, popupIconSize, 0, NULL, DI_NORMAL); //DI_DEFAULTSIZE
			r.left = popupTextOffset+popupIconSize+4;
		} else
			r.left = popupTextOffset;
    	r.right = backBMP.x-popupTextOffset;
    	r.top= totalHeight;
    	r.bottom = totalHeight + titleHeight;
    	DrawText(DC, iTemp->name, lstrlen(iTemp->name), &r, DT_VCENTER|DT_SINGLELINE|DT_NOCLIP);

    	if (!noPopupBevel)
    	{
			paintPopupBevel(this->hwnd, iTemp, totalHeight, this->menuWidth);
    	}
		totalHeight = totalHeight + popupSubHeight;
    	iTemp = iTemp->next;
    }
	if (bottomBMP.bitmap) {
		if (!noPopupPix) {
			SelectObject(memDC, bottomBMP.bitmap);
			if (shrinkPopupBar) {
				StretchBlt(DC,0,totalHeight, menuWidth, popupSubHeight, memDC, 0, 0,bottomBMP.x, bottomBMP.y, SRCCOPY);
			} else {
				for(int x=0; x<menuWidth; x+=bottomBMP.x)
					for(int y=totalHeight+1; y<(totalHeight+popupSubHeight); y+=bottomBMP.y) {
						bWidth = x+bottomBMP.x<menuWidth?bottomBMP.x:x<menuWidth?menuWidth-x:menuWidth;
						bHeight = y+bottomBMP.y<(totalHeight+popupSubHeight)?bottomBMP.y:y<popupSubHeight?popupSubHeight-y:popupSubHeight;
						BitBlt(DC, x, y, bWidth, bHeight, memDC, 0, 0, SRCCOPY);
					}
			}
		} else {
			hBrush = CreateSolidBrush(titleBack);
			SetRect(&r, 0, totalHeight, menuWidth, totalHeight+titleHeight);
			FillRect(DC, &r, hBrush);
			DeleteObject(hBrush);
		}
	}	
	SelectObject(DC, oldFont);
   	ReleaseDC(this->hwnd, DC);
    EndPaint(this->hwnd,&ps);	

    //displays the window
    DeleteDC(memDC);
}

void Popup::closeAll() {
	if (!this->pinned)
		ShowWindow(this->hwnd, SW_HIDE);
/* _TAG_	sent to LS to have the desktop know the popups are hidden */
	SendMessage(parentWin, LM_POPUPHIDDEN, NULL, NULL);
/* _TAG_ END */
}

menuItem *Popup::keyItem(int c) {
	menuItem *temp;

	if (this->isSelected != NULL) {
		if (this->isSelected->next != NULL) {
			temp = this->isSelected->next;
		} else {
			temp = this->root;
		}
	} else {
		temp = this->root;
	}
	while (temp != NULL) {
		if (temp->name[0] == c) {
			return temp;
		}
		temp = temp->next;
	}
	temp = this->root;
	while (temp != this->isSelected) {
		if (temp->name[0] == c) {
			return temp;
		}
		temp = temp->next;
	}
	return NULL;
}
////////////////////////////////////////////////////////////////////////////////
//END CLASS DEFINITION 
////////////////////////////////////////////////////////////////////////////////

menuItem *GetItem(menuItem *root, int num) {
	menuItem *temp = root;
	while (temp != NULL) {
		if (temp->itemNo == num)
			break;
		temp = temp->next;
	}
	return temp;
}

//cleanup, not much to do
int quitModule(HINSTANCE dllInst)
{
    DestroyWindow(hMainWnd);
    DeleteObject(popupFont);
	if (titleBMP.bitmap)
		DeleteObject(titleBMP.bitmap);
	if (backBMP.bitmap)
		DeleteObject(backBMP.bitmap);
	if (selectBMP.bitmap)
		DeleteObject(selectBMP.bitmap);
	if (bottomBMP.bitmap)
		DeleteObject(bottomBMP.bitmap);
	if (showPopupIcons) {
		DeleteObject(popupDefaultIcon);
	}
    UnregisterClass(szAppName,dllInst); // unregister window class
    return 1;
}

//DLL entry point
int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
    appInstance = dllInst;
    parentWin = ParentWnd;

    //set all menus to NON-selected mode
    //for(int SE = 0; SE < 255; selected[SE] = FALSE, SE++);
    //parse file and fill popup class structure
    initializePopupMenu();
    
    memset(&wc,0,sizeof(wc));
    wc.lpfnWndProc = (WNDPROC)&Popup::wndProc;// our window procedure
    //wc.hbrBackground = (HBRUSH) GetStockObject(LTGRAY_BRUSH);
    wc.hInstance = appInstance;			
    wc.lpszClassName = szAppName;// our window class name
    wc.style = 0;

    if (!RegisterClass(&wc)) 
    {
    	MessageBox(NULL,"Error registering window class",szAppName,MB_OK);
    	return 1;
    }
    hMainWnd = CreateWindowEx(
        	WS_EX_TOPMOST|WS_EX_TOOLWINDOW,//WS_EX_TRANSPARENT,									// exstyles 
    		szAppName,								// our window class name
    		"",											// use description for a window title
    		WS_POPUP,				
    		0, 0,									// position 
    		popup.menuWidth,(popup.totalHeight + (titleHeight*2)),				// width & height of window
    		ParentWnd,								// parent window 
    		NULL,									// no menu
    		appInstance,								// hInstance of DLL
    		NULL);									// no window creation data

    SetWindowLong(hMainWnd, GWL_USERDATA, magicDWord);
	if (transBlt && !noPopupPix) {
		SetWindowRgn(hMainWnd, popup.region, FALSE);
	}
	popup.hwnd = hMainWnd;

    //create the message id's that popup.dll will use
    int Msgs[10];
    Msgs[0] = LM_LPOPUP;
    Msgs[1] = LM_HIDEPOPUP;
    Msgs[2] = LM_GETREVID;
	Msgs[3] = 0;
    SendMessage (ParentWnd, LM_REGISTERMESSAGE, (WPARAM)hMainWnd, (LPARAM)Msgs);
	
	for (int i=0; i<256; i++)
		wentRight[i] = FALSE;
    return 0;
}

void paintPopupBevel(HWND hwnd, menuItem* item, int totalHeight, int width)
{
	HDC hDC;
	HPEN hPen;
	HPEN hOldPen;

	hDC = GetDC(hwnd);
	// First we draw the white frame on the left/top
	hPen = CreatePen(PS_SOLID, 1, RGB(255,255,255));
	hOldPen = (HPEN) SelectObject(hDC, hPen);
	MoveToEx(hDC, 0, totalHeight, NULL);
	LineTo(hDC, width, totalHeight);
	MoveToEx(hDC, 0, totalHeight, NULL);
	LineTo(hDC, 0, totalHeight+popupSubHeight-1);
	SelectObject(hDC, hOldPen);
	DeleteObject(hPen);

	// Now the black frame on the bottom/right
	hPen = CreatePen(PS_SOLID, 1, RGB(0,0,0));
	hOldPen = (HPEN) SelectObject(hDC, hPen);
	MoveToEx(hDC, width-1, totalHeight, NULL);
	LineTo(hDC, width-1, totalHeight+popupSubHeight-1);
	MoveToEx(hDC, 0, totalHeight+popupSubHeight-1, NULL);
	LineTo(hDC, width-1, totalHeight+popupSubHeight-1);
	SelectObject(hDC, hOldPen);
	DeleteObject(hPen);
	ReleaseDC(hwnd, hDC);
}

void executeItem(menuItem* pItem)
{
	if (pItem->command == FALSE)
		return;
	windowType *winList;
	HWND hwnd;
	char title[MAX_PATH];
	int maxWin;
	winList = (windowType *)SendMessage(parentWin, LM_WINLIST, 0, 0);
	maxWin = (int) SendMessage(parentWin, LM_WINLIST, 1, 0);
	for (int n = 0; n < maxWin && winList[n].Handle; n++) {
		hwnd = winList[n].Handle;
		GetWindowText(hwnd, (char *)&title, MAX_PATH);
		if (!strcmp(pItem->name, (char *)&title)){
			if (IsIconic(hwnd))
				OpenIcon(hwnd);
			SetForegroundWindow(hwnd);
			break;
		}
	}
}


void initializePopupMenu() 
{
	char iname[MAX_PATH];
    
    GetRCString("HotListName", popupMenuName, "", 256);
    GetRCString("PopupTitlePix", titleBMP.name	, "", 256);
    GetRCString("PopupEntryPix", backBMP.name	, "", 256);
    GetRCString("PopupSelEntryPix", selectBMP.name	, "", 256);
	GetRCString("PopupBottomPix", bottomBMP.name, "", 256);
    GetRCString("PopupFontFace", popupFontFace, "Arial", 256);
	if (strlen(titleBMP.name)==0 || strlen(backBMP.name)==0 || strlen(selectBMP.name)==0) 
		noPopupPix = TRUE;
	popupFolderIcon = GetRCBool("NoPopupFolderIcon", FALSE);
	transBlt = GetRCBool("NoPopupTransparent", FALSE);
    noPopupBevel = GetRCBool("NoPopupBevel", TRUE);
    popupFontHeight = GetRCInt("PopupFontHeight", 16);
    titleColor = GetRCColor("PopupTitleColor", 0xffffff);
    normColor = GetRCColor("PopupEntryColor", 0x000000);
    selColor = GetRCColor("PopupSelEntryColor", 0x000000);
    popupMenuDelay = GetRCInt("PopupMenuDelay",0);
    popupMinWidth = GetRCInt("minpopupwidth", 100);
    popupSubHeight = GetRCInt("PopupSubmenuHeight", 20);
	titleHeight = GetRCInt("PopupSubmenuHeight", 20);
	popupFixedWidth = GetRCInt("PopupFixedWidth", 0);
	isFixedWidth = popupFixedWidth==0?FALSE:TRUE;
	popupTextOffset = GetRCInt("PopupTextOffset", 5);
	showPopupIcons = GetRCBool("ShowPopupIcons", TRUE);
	if (showPopupIcons) {
		popupIconSize = GetRCInt("PopupIconSize", 20);
		GetRCString("PopupDefaultIcon", iname, "default.ico", 256);
		popupDefaultIcon = LoadLSIcon(iname, NULL);
		popupIconSize = popupIconSize>popupSubHeight?popupSubHeight:popupIconSize;
		popupTop = (popupSubHeight-popupIconSize)/2;
	} else 
		popupIconSize = 0;
	// Bottom bitmap is loaded as there is no setting to indicate whether it is drawn or not - yet ...
	bottomBMP.bitmap = LoadLSImage(bottomBMP.name, NULL);
	if (!noPopupPix) {
	    //loads up images i will eventualy seperate this into its own function
		backBMP.bitmap = LoadLSImage(backBMP.name, NULL);
	    titleBMP.bitmap = LoadLSImage(titleBMP.name, NULL);
		selectBMP.bitmap = LoadLSImage(selectBMP.name, NULL); 

	    GetLSBitmapSize(selectBMP.bitmap, &selectBMP.x, &selectBMP.y);
		GetLSBitmapSize(backBMP.bitmap, &backBMP.x, &backBMP.y);
	    GetLSBitmapSize(titleBMP.bitmap, &titleBMP.x, &titleBMP.y);
		GetLSBitmapSize(bottomBMP.bitmap, &bottomBMP.x, &bottomBMP.y);

		//mian's transparency stuff
		selectBMP.region = BitmapToRegion(selectBMP.bitmap, RGB(255,0,255), 0x10101010, 0, 0);
		titleBMP.region = BitmapToRegion(titleBMP.bitmap, RGB(255,0,255), 0x10101010, 0, 0);
		backBMP.region = BitmapToRegion(backBMP.bitmap, RGB(255,0,255), 0x10101010, 0, 0);
		bottomBMP.region = BitmapToRegion(bottomBMP.bitmap, RGB(255,0,255), 0x10101010, 0, 0); 
		//Bitmap shrink/tile option
		shrinkPopupBar = GetRCBool("ShrinkPopupBar", TRUE);
	} else {
	    titleBack = GetRCColor("PopupTitleBack", 0x000000);
            normBack = GetRCColor("PopupNormalBack", 0xa4a0a0);
	    selBack = GetRCColor("PopupSelBack", 0xffffff);
	}

    popupFont = CreateFont(
    		popupFontHeight,
    		0,
    		0,
    		0,
    		FW_NORMAL,
    		FALSE,
    		FALSE,
    		FALSE,
    		DEFAULT_CHARSET,
    		OUT_DEFAULT_PRECIS,
    		CLIP_DEFAULT_PRECIS,
    		DEFAULT_QUALITY,
    		DEFAULT_PITCH,
    		popupFontFace);
	buildTaskList();
}

void buildTaskList() {
	windowType *winList;
	char title[MAX_PATH];
	BOOL noList = TRUE;
	int n, maxWin;

	popup.delAllMenuItems();;
	winList = (windowType *)SendMessage(parentWin, LM_WINLIST, 0, 0);
	maxWin = (int) SendMessage(parentWin, LM_WINLIST, 1, 0);
	for (n = 0; n < maxWin && winList[n].Handle; n++)
	{
		HICON hIcon = NULL;
		noList = FALSE;

		SendMessageTimeout(winList[n].Handle, WM_GETICON, 1, 0, 0, 250, (unsigned long *)&hIcon);
		if (!hIcon) 
			SendMessageTimeout(winList[n].Handle, WM_QUERYDRAGICON, 0, 0, 0, 1000, (unsigned long *)&hIcon);
		if (!hIcon) 
			hIcon = (HICON)GetClassLong(winList[n].Handle, GCL_HICON);
		if (!hIcon) 
			hIcon = (HICON)GetClassLong(winList[n].Handle, GCL_HICONSM);
	    popup.newMenuItem();
		title[0] = 0;
		if (winList[n].Handle != NULL) {
			GetWindowText(winList[n].Handle, (char *)&title, MAX_PATH);
		    strcpy(popup.current->name, title);
			popup.current->hIcon = hIcon;
			popup.current->command = TRUE;
		//	pop->current->hwnd = winList[n].Handle;
		}
	}
	if (noList) {
		popup.newMenuItem();
    	strcpy(popup.current->name, "No Tasks");
		popup.current->command = FALSE;
	}
	setPopupValues();
}

void setPopupValues() {
    menuItem *counter = popup.root;
	popup.totalHeight = 0;
	int maxWidth = popupMinWidth;
	SIZE name;
	HDC popDC = CreateDC("DISPLAY", NULL, NULL, NULL);
	SelectObject(popDC, popupFont);
	int count = 1;
	while(counter != NULL)
    {
		popup.totalHeight = popupSubHeight + popup.totalHeight;
		GetTextExtentPoint32(popDC,(LPCTSTR)&counter->name,strlen(counter->name),&name);
		if (name.cx+(popupTextOffset*2)+popupIconSize+15 > maxWidth)
			maxWidth = name.cx+(popupTextOffset*2)+popupIconSize+15;
		counter->itemNo = count;
		counter = counter->next;
		count++;
	}
	popup.menuWidth = isFixedWidth?popupFixedWidth:maxWidth;
	if (transBlt && !noPopupPix) {
		popup.region = GetRegion(popup.root);
	}
	// Set close button area for pinning
	popup.close.left = popup.menuWidth - 11;
	popup.close.top = 3;
	popup.close.right = popup.menuWidth - 3;
	popup.close.bottom = 11;
	DeleteDC(popDC);
}

HRGN GetRegion(menuItem *item) {
	menuItem *temp;
	HRGN mainRgn, clipRgn;
	int height = titleHeight;

	mainRgn = CreateRectRgn(0, 0, 0, 0);
	CombineRgn(mainRgn, mainRgn, titleBMP.region, RGN_OR);
	temp = item;
	while (temp != NULL) {
		clipRgn = CreateRectRgn(0, 0, 0, 0);
		CombineRgn(clipRgn, backBMP.region, NULL, RGN_COPY);
		OffsetRgn(clipRgn, 0, height);
		CombineRgn(mainRgn, mainRgn, clipRgn, RGN_OR);
		DeleteObject(clipRgn);
		height = height + popupSubHeight;
		temp = temp->next;
	}
	if (bottomBMP.bitmap) {
		clipRgn = CreateRectRgn(0, 0, 0, 0);
		CombineRgn(clipRgn, bottomBMP.region, NULL, RGN_COPY);
		OffsetRgn(clipRgn, 0, height);
		CombineRgn(mainRgn, mainRgn, clipRgn, RGN_OR);
		DeleteObject(clipRgn);
	}
	return mainRgn;
}

void displayPopup(HWND hwnd, int x, int y)
{
	int height;
	height = bottomBMP.bitmap?titleHeight*2:titleHeight;
	popup.onLeft = FALSE;
    
    //Get popup coordinates
	if (x + popup.menuWidth > GetSystemMetrics(SM_CXSCREEN)) {
		x -= popup.menuWidth;
		popup.onLeft = TRUE;
	}

	if ((y + popup.totalHeight + height) > GetSystemMetrics(SM_CYSCREEN))
		y = GetSystemMetrics(SM_CYSCREEN) - (popup.totalHeight + height);
	SetWindowPos(hwnd, HWND_TOP, x, y, popup.menuWidth,(popup.totalHeight + height), 
    	SWP_SHOWWINDOW);
    popup.paintPopup();
}

void keyedMenu(HWND menu, menuItem *showItem) {
	RECT lpMenu;
	int height;

	height=showItem->itemNo*popupSubHeight;
	GetWindowRect(menu, &lpMenu);
	SetCursorPos(lpMenu.left+2, lpMenu.top+height+2);
}

/*
	$Log: popup.cpp,v $
	Revision 0.1 1999/03/01 14:21:26 _TAG_
	
	Left click popup. Shows only active tasks
	
*/
