 /*

  This is a part of the LiteStep Shell Source code.

  Copyright (C) 1997-98 The LiteStep Development Team

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


/****************************************************************************
 12/20/98 - Fahim Farook
			Perfected subfolder pinning via a double-click on the titlebar and
			pinned popup closing via a click on the close button
 12/19/98 - Fahim Farook
			Removed the subList structure and modified popup navigation to use a
			linked list created via parent & child relationships
 12/10/98 - Fahim Farook
			Rewrote most of the popup code so that it used a single popup structure
			and was more object oriented
 12/02/98 - Fahim Farook
			Added fixed width popup menu support via PopupFixedWidth in step.rc
 12/01/98 - Fahim Farook
			Fixed the popups crashing on moving cursor to bottom pic bug
 11/24/98 - Edwin Man
            Changed default PopupNormalColor to A4A0A0; looks better
 11/24/98 - Fahim Farook
			Changed the popup navigation code to use standardized item heights
			so that the code is more readable and faster
 11/23/98 - Fahim Farook
			Streamlined the transparency code and made it setup and load the 
			region at start up then just set the region when a popup is created
 11/21/98 - Fahim Farook
			Added a working close button to the main popup when pinned
			Modified keyboard navigation so that left and right key functionality
			switches when the current folder is to the left of it's parent
 11/19/98 - Fahim Farook
			Modified popup displaying so that if a popup opens on the left all 
			subfolders will keep on opening to the left so as to avoid the
			clutter that resulted in earlier implementations
			Added the ability to specify colours instead of bitmaps for popup
			background via PopupTitleBack, PopupNormBack & PopupSelBack in
			step.rc
 11/18/98 - Fahim Farook
			Modified icon extraction based on code by Johan Redestig to make it
			more efficient and added an icon deletion routine when quitting to 
			free up resources.
 11/14/98 - Fahim Farook
			Added popup icons via ShowPopupIcons in step.rc and allowed customization
			via PopupDefaultIcon and PopupIconSize
 11/12/98 - Fahim Farook
			Added popup pinning functionality to the main popup and changed
			mouse click responses from LBUTTONDOWN to LBUTTONUP
 11/09/98 - Fahim Farook
			Merged in popup transparency code provided by mian and took out the
			old transparentBltLS stuff
 11/08/98 - Fahim Farook
			Added dynamic popup width sizing support & dynamic folder support
			via the !PopupDynamicFolder command
 11/08/98 - J Redestig
			Sets working directory. Added three utility functions for cleaner
			code (paintFolderArrow, paintPopupBevel, executeItem). Removed
			some flicker from displaySingleItem
 10/31/98 - Fahim Farook
			Added support for popup menu navigation via keyboard
 10/29/98 - Cyberian (Fahim Farook)
			Added support for popupmenu height customization. The title bar
			wouldn't change sizes earlier - now takes its value from PopupSubMenuHeight
			in step.rc
 09/11/98 - M. West
    		New popups created, limited functionality, will parse in a simple
    		1 layer popup menu, and run programs from it,
 30/10/98 - B. Kilian
			Fixed Fixed the popups repainting problem, and the setWindowPos 
			in the paint.
 30/10/98 - J Redestig
			Setforegroundwindow, so the popups get focus correctly, and <ESC> exits popups.
 30/10/98 - T Engel
			Environment substitution for !PopupFolders

****************************************************************************/
#include <windows.h>
#include <stdio.h>
#include <shlobj.h>
#include <tchar.h>
#include <process.h>
#include "popup.h"

const char rcsRevision[] = "$Revision: 1.1 $"; // Our Version 
const char rcsId[] = "$Id: popup.cpp,v 1.1 1999/01/20 22:51:26 cyberian Exp $"; // The Full RCS ID.
char popupFontFace[256];
char popupMenuName[256];
const char szAppName[] = "PopupMenu"; // Our window class, etc

int popupSubHeight = 20;
int popupMinWidth = 100;
int popupIconSize = 20;
int titleHeight = 20;
int popupFixedWidth = 0;
int popupLevel = 0;
int popupMenuDelay = 0;
int popupFontHeight;
int popupTextOffset;
int popupTop;

menuItem *previousItem = NULL;	
menuItem *timerPointer = NULL;

HFONT popupFont;
HICON popupDefaultIcon;
COLORREF titleColor, selColor, normColor;
COLORREF titleBack,selBack, normBack;
WNDCLASS wc;

Popup popup; //popup class struct
fileItem *popupFile, *treeItem;  //rc file info holder

HINSTANCE appInstance;
HWND itemCaller = NULL;
HWND parentWin;
HWND hMainWnd = NULL;

BOOL wentRight[255];
BOOL noPopupBevel;
BOOL noPopupPix = FALSE;
BOOL transBlt=TRUE;
BOOL popupFolderIcon=TRUE;
BOOL showPopupIcons = FALSE;
BOOL isFixedWidth = FALSE;
BOOL shrinkPopupBar = FALSE;

BitMP backBMP;
BitMP titleBMP;
BitMP selectBMP;
BitMP bottomBMP;

int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath);
int fillDirectory(char *dirName, menuItem *target);
void fillThisDir(char *dirName, menuItem *target);
void initializePopupMenu();
void displayPopup(HWND hwnd, int x, int y);
void paintFolderArrow(HWND hwnd, menuItem *item, int totalHeight, int width);
void paintPopupBevel(HWND hwnd, menuItem* item, int totalHeight, int width);
void executeItem(menuItem* pItem);
void keyedSubMenu(HWND Menu, menuItem *showItem);
void GetItemIcon(void *item);
void RegistryLookup(char* subKey, char *value);
void buildPopupTree(Popup *pop);
void buildTaskList(menuItem *target);
void setPopupValues(Popup *pop);
menuItem *GetItem(menuItem *root, int num);
HRGN GetRegion(menuItem *item);

////////////////////////////////////////////////////////////////////////////////
//BEGIN CLASS DEFINITION 
////////////////////////////////////////////////////////////////////////////////

menuItem::~menuItem() {
   	if(next != NULL)
   		delete next;
   	if(SMenu != NULL)
   		delete SMenu;
	if (hIcon != NULL)
		DeleteObject(hIcon);
}

int Popup::returnMenuWidth()
{
    return(menuWidth);
}


fileItem::fileItem()
{
    strcpy(itemName, "\0");
    strcpy(bmpName, " ");
    strcpy(popupCommand, " ");
    next = NULL;
    itemHeight = popupSubHeight;
}

fileItem::~fileItem()
{
    delete next;
}


BOOL Popup::newMenuItem()
{
    menuItem *temp = root;
	menuItem *tprv = NULL;
    
    while(1) //finds the last node in the list
    {
    	if(temp == NULL)
    		break;
		else if(temp->next == NULL) {
			tprv = temp;
    		break;
		} else {
			tprv = temp;
			temp = temp->next;
		}
    }
    
    if(temp == NULL)
    {
    	root = new menuItem;
    	current = root;
    }
    else 
    {
    	temp->next = new menuItem;
    	current = temp->next;
		current->prev = tprv;
    }
    
    return(TRUE);
}

// window procedure for our window
LRESULT CALLBACK Popup::wndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int minWidth, totHeight;
	HWND hwndGetFocus;
	POINT pt;
    RECT r;
    menuItem *item;
	Popup *pop;

   	GetCursorPos(&pt);
   	GetWindowRect(hwnd, &r);
   	pt.x -= r.left;
   	pt.y -= r.top;
	
	pop = getPopup(hwnd);
	if (pop == NULL)
		pop = &popup;
	minWidth = pop->menuWidth;
	totHeight = pop->totalHeight+titleHeight;
	
    switch (message) {
	case LM_GETREVID: {
		char *buf = (char *) lParam;

		if (wParam == 0) {
			strcpy(buf, "popup.dll: ");
			strcat(buf, &rcsRevision[11]);
			buf[strlen(buf)-1] = '\0';
		} else if (wParam == 1) {
			strcpy(buf, &rcsId[1]);
			buf[strlen(buf)-1] = '\0';
		} else {
			strcpy(buf, "");
		}
		return strlen(buf);
	}

	case LM_POPUP:
		GetCursorPos(&pt);
		displayPopup(hwnd, pt.x, pt.y);
		SetForegroundWindow(hwnd);
		return 0;
	case WM_PAINT:
		pop->paintPopup();
		if (pop->pinned) {
			DrawFrameControl(GetDC(hwnd), &pop->close, DFC_CAPTION, DFCS_CAPTIONCLOSE|DFCS_FLAT);
		}
		break;

	case WM_ERASEBKGND:
		return 0;

    case WM_TIMER:
    	KillTimer(hwnd, 1);
    	if(timerPointer == NULL)
    		break;

    	if(timerPointer->SMenu != pop && timerPointer != NULL && hwnd == itemCaller)
    	{
   			pop->closeSubMenus();
			pop->isSubOpen = TRUE;
    		pop->createWindow(pop, timerPointer, timerPointer->itemNo*popupSubHeight);
    		timerPointer = NULL;			
    	}
    	break;

    case WM_MOUSEMOVE: 
	{
	    int y = HIWORD(lParam); 
		int prevHeight;
		int itemNo = y / popupSubHeight;
		
		item = GetItem(pop->root, itemNo);
		if (previousItem == item)
			break;
		if(y < titleHeight || y>=totHeight) {
			// Do nothing
		} else {
			if (previousItem != NULL) {
				prevHeight = (previousItem->itemNo*popupSubHeight);
				if(y < prevHeight || y > prevHeight || previousItem->itemLevel > item->itemLevel)
    			{
					pop->closeSubMenus();
					timerPointer = NULL;
					if(pop->isSelected != NULL) {
						pop->isSubOpen = FALSE;
		    			pop->selectItem(FALSE);
					}
    			}
			}
			pop->isSelected = item;
			pop->selectItem(TRUE);
			previousItem = item;
    		if(item->SMenu != NULL && !item->SMenu->pinned) {
   				if(timerPointer != item)
   				{
   					SetTimer(hwnd, 0, popupMenuDelay, NULL);
   					timerPointer = item;
					itemCaller = hwnd;
   				}
   			}
   		}
    	break;
	}
    case WM_NCHITTEST:
    	if(pt.y < titleHeight ) {
			if (pop->pinned) {
				if (PtInRect(&pop->close, pt)) {
					return (HTCLOSE);
				} else
					return (HTCAPTION);
			} else
   				return (HTCAPTION);
	   	}
   		else break;

    case WM_NCLBUTTONDBLCLK:
		if (!pop->pinned) {
			pop->pinned = TRUE;
			if (pop != &popup) {
				pop = &popup;
				pop->closeSubMenus();
				ShowWindow(pop->hwnd, SW_HIDE );
			} else
				pop->closeSubMenus();
			InvalidateRect(hwnd, NULL, FALSE);
		} else {
			pop->pinned = FALSE;
			if (pop->parent == NULL) {
				InvalidateRect(hwnd, NULL, FALSE);
			} else {
				pop->closeSubMenus();
				pop->parent->child = NULL;
				DestroyWindow(pop->hwnd);
			}
		}
		break;

	case WM_NCLBUTTONDOWN:
		if (pop->pinned) {
			if (PtInRect(&pop->close, pt)) {
				pop->pinned = FALSE;
				pop->closeSubMenus();
				if (pop->parent == NULL)
					ShowWindow(pop->hwnd, SW_HIDE);
				else {
					pop->parent->child = NULL;
					DestroyWindow(pop->hwnd);
				}
			}
		}
		break;
/* _TAG_	handles this message which is sent by the desktop */
	case LM_HIDEPOPUP:
		closeAll(&popup);
		previousItem = NULL;
		return 0;
/* _TAG_ END */
    case WM_ACTIVATE:
		{
		    WORD fActive;
			fActive = LOWORD(wParam);

    		if(fActive == WA_INACTIVE) {
	    		hwndGetFocus = (HWND)lParam; /* handle of window receiving focus */
				if (hwndGetFocus == NULL || getPopup(hwndGetFocus) == NULL) {
					closeAll(&popup);
					previousItem = NULL;
				}
			}
			return 0;
	    }
    	break;

    case WM_LBUTTONUP:
		if (pt.y>titleHeight && pt.y<=totHeight) {
	  		if(pop->isSelected != NULL)
   			{
				item = pop->isSelected;
   				if(item->SMenu == NULL )
	   			{
   					if(item->command[0] != '!')
   					{
						executeItem(item);
					} else
   						ParseBangCommand (hwnd, item->command, item->params);
					pop = &popup; // Put in code later to verify open popup
		    		pop->closeSubMenus();
					if (!pop->pinned) {
						previousItem = NULL;
						pop->isSelected = NULL;
						if (pop->parent == NULL)
    						ShowWindow(pop->hwnd, SW_HIDE );
						else
							DestroyWindow(pop->hwnd);
					}
   					break;
				}
			}
		}
    	break;
	case WM_KEYDOWN: 
		switch (wParam) {
		case VK_ESCAPE:
			closeAll(&popup);
			previousItem = NULL;
			break;

		case VK_DOWN: 
			if (!pop->wentRight && pop->parent != NULL) {
				pop = pop->parent;
			}
			if (pop->isSelected == NULL || pop->isSelected == pop->current) {
				keyedSubMenu(pop->hwnd, pop->root);
			} else {
				keyedSubMenu(pop->hwnd, pop->isSelected->next);
			}
			break;

		case VK_UP: 
			if (!pop->wentRight && pop->parent != NULL) {
				pop = pop->parent;
			}
			if (pop->isSelected == NULL || pop->isSelected == pop->root) {
				keyedSubMenu(pop->hwnd, pop->current);
			} else {
				keyedSubMenu(pop->hwnd, pop->isSelected->prev);
			}
			break;

		case VK_RIGHT: 
			if (!pop->onLeft) {
				if (pop->wentRight) {
					if (pop->isSelected->SMenu != NULL && pop->isSelected != NULL) { 
						keyedSubMenu(pop->child->hwnd, pop->isSelected->SMenu->root);
					}
				} else {
					keyedSubMenu(pop->hwnd, pop->root);
					pop->wentRight = TRUE;
				}
			} else {
				// Key left functionality for when menu's on the left
				if (pop->wentRight) {
					pop->wentRight = FALSE;
					if (pop->parent != NULL) {
						keyedSubMenu(pop->parent->hwnd, pop->parent->isSelected);
					}
				} else {
					if (pop->parent != NULL) {
						if (pop->parent->parent != NULL) {
							pop = pop->parent->parent;
							pop->parent->wentRight = FALSE;
							keyedSubMenu(pop->hwnd, pop->isSelected);
						}
					}
				}
			}
			break;
		case VK_LEFT: //Sub folder
			if (!pop->onLeft) {
				if (pop->wentRight) {
					pop->wentRight = FALSE;
					if (pop->parent != NULL) {
						keyedSubMenu(pop->parent->hwnd, pop->parent->isSelected);
					}
				} else {
					if (pop->parent != NULL) {
						if (pop->parent->parent != NULL) {
							pop = pop->parent->parent;
							if (pop->parent != NULL)
								pop->parent->wentRight = FALSE;
							keyedSubMenu(pop->hwnd, pop->isSelected);
						}
					}
				}
			} else {
				// Key right functionality for when menu's on the left
				if (pop->wentRight) {
					if (pop->isSelected->SMenu != NULL && pop->isSelected != NULL) { 
						keyedSubMenu(pop->child->hwnd, pop->isSelected->SMenu->root);
					}
				} else {
					keyedSubMenu(pop->hwnd, pop->root);
					pop->wentRight = TRUE;
				}
			}
			break;
		case VK_RETURN: //Sub folder
			if (pop->isSelected != NULL) {
	    		if(pop->isSelected->SMenu == NULL) {
	    			char temp[256];
					strcpy(temp, pop->isSelected->command);
	    			if(pop->isSelected->command[0] != '!') {
						executeItem(pop->isSelected);
					} else
	    				ParseBangCommand (hwnd, pop->isSelected->command, pop->isSelected->params);
					closeAll(&popup);
					previousItem = NULL;
				}
			} 
			break;
		default:
			if (isalnum(wParam)) {
				if (!pop->wentRight && pop->parent != NULL)
					pop = pop->parent;
				item = pop->keyItem(wParam);
				if (item != NULL) 
					keyedSubMenu(pop->hwnd, item);
			}
		}
		break;
	}
    return DefWindowProc(hwnd, message, wParam, lParam);
}

void Popup::createWindow(Popup *pop, menuItem *firstItem, int height) {
    RECT lpRect;
	int prevWidth, tHeight, bHeight;
	HRGN tempRgn;
	HWND calledFrom, thisWin;
	BOOL onLeft;
	Popup *win;

	calledFrom = pop->hwnd;
	tHeight = bottomBMP.bitmap?titleHeight*2:titleHeight;
	bHeight = bottomBMP.bitmap?titleHeight:0;
    GetWindowRect(calledFrom, &lpRect);
	win = firstItem->SMenu;
	win->parent = pop;
	pop->child = win;
	prevWidth = pop->menuWidth;
	onLeft = pop->onLeft;
	if (!onLeft) {
		if (lpRect.right + win->menuWidth > GetSystemMetrics(SM_CXSCREEN)) { 
    		lpRect.right = lpRect.right - win->menuWidth - prevWidth;
			win->onLeft = TRUE;
		}
	} else {
		if (lpRect.right - win->menuWidth - prevWidth < 0) {
			win->onLeft = FALSE;
		} else {
			lpRect.right = lpRect.right - win->menuWidth - prevWidth;
			win->onLeft = TRUE;
		}
	}

	if ((win->totalHeight+lpRect.top+height+bHeight) > GetSystemMetrics(SM_CYSCREEN))
		lpRect.top = GetSystemMetrics(SM_CYSCREEN) - (win->totalHeight+height+bHeight);

    thisWin = CreateWindowEx(
        	WS_EX_TOPMOST|WS_EX_TOOLWINDOW,			// exstyles 
    		szAppName,								// our window class name
    		"",										// use description for a window title
    		WS_POPUP,				
			lpRect.right, (lpRect.top+height), 							// position 
    		win->menuWidth,(win->totalHeight)+tHeight,	// width & height of window
    		NULL,									// parent window 
    		NULL,									// no menu
    		appInstance,							// hInstance of DLL
    		NULL);									// no window creation data

    win->hwnd = thisWin;
    SetWindowLong(thisWin, GWL_USERDATA, magicDWord);
	if (transBlt && !noPopupPix) {
		tempRgn = CreateRectRgn(0, 0, 0, 0);
		CombineRgn(tempRgn, win->region, NULL, RGN_COPY);
		SetWindowRgn(thisWin, tempRgn, FALSE);
	}
	BOOL temp2 = SetWindowPos(thisWin, HWND_TOP, lpRect.right, (lpRect.top+height-titleHeight), 
    win->menuWidth,win->totalHeight+tHeight,
    SWP_SHOWWINDOW);
	win->paintPopup();
}

void Popup::closeSubMenus() {
	int level;
	Popup *temp, *stay;

	stay = this;
	level = stay->root->itemLevel;
	temp = stay;
	while (1) {
		if (temp->isSelected !=NULL && temp->isSubOpen) {
			if (temp->isSelected->SMenu != NULL) {
				temp = temp->isSelected->SMenu;
			} else
				break;
		} else
			break;
	}
	while (temp != stay) {
		if (!temp->pinned) {
			DestroyWindow(temp->hwnd);
			temp->hwnd = NULL;
			temp->onLeft = FALSE;
			if (temp->parent != NULL) {
				temp->parent->child = NULL;
			}
			temp->wentRight = FALSE;
			temp->isSubOpen = FALSE;
		}
		temp = temp->parent;
	}
}

void Popup::selectItem(BOOL selected)
{
	HWND hwnd;
	menuItem *item;
    HDC DC, memDC;
    PAINTSTRUCT ps;
    HFONT oldFont;
	HBRUSH hBrush;
    RECT r;
	int totalHeight, bWidth, bHeight;
	static menuItem *paintedLastTime = NULL;
	static BOOL bWasSelected = FALSE;

	item = this->isSelected;
	totalHeight = item->itemNo*popupSubHeight;
	hwnd = this->hwnd;
	// Test if it is the same menu item thet we painted last time
	// is so return, this will reduce flicker.
	if(paintedLastTime == item && bWasSelected == selected && previousItem != NULL)
		return;
	else{
		bWasSelected = selected;
		paintedLastTime = item;
	}
	int minWidth;
	minWidth = this->menuWidth;

   	DC = BeginPaint(hwnd,&ps);
   	memDC = CreateCompatibleDC(DC);
   	DC = GetDC(hwnd);
   	oldFont = (HFONT) SelectObject(DC, popupFont);

    if(selected) {
		if (!noPopupPix) {
		   	SelectObject(memDC, selectBMP.bitmap);
//   			BitBlt(DC,0,totalHeight, selectBMP.x, popupSubHeight, memDC, 0, 0,SRCCOPY);
			if (shrinkPopupBar) {
				StretchBlt(DC,0,totalHeight, menuWidth, popupSubHeight, memDC, 0, 0,selectBMP.x, selectBMP.y, SRCCOPY);
			} else {
				for(int x=0; x<menuWidth; x+=selectBMP.x)
					for(int y=totalHeight+1; y<(totalHeight+popupSubHeight); y+=selectBMP.y) {
						bWidth = x+selectBMP.x<menuWidth?selectBMP.x:x<menuWidth?menuWidth-x:menuWidth;
						bHeight = y+selectBMP.y<(totalHeight+popupSubHeight)?selectBMP.y:y<popupSubHeight?popupSubHeight-y:popupSubHeight;
						BitBlt(DC, x, y, bWidth, bHeight, memDC, 0, 0, SRCCOPY);
					}
			}
		} else {
			hBrush = CreateSolidBrush(selBack);
			SetRect(&r, 0, totalHeight,minWidth, totalHeight+titleHeight);
			FillRect(DC, &r, hBrush);
			DeleteObject(hBrush);
		}
    	SetTextColor(DC,  selColor);
	   	SetBkMode(DC, TRANSPARENT);
    	r.right = selectBMP.x-popupTextOffset;
    } else {
		if (!noPopupPix) {
		   	SelectObject(memDC, backBMP.bitmap);
			if (shrinkPopupBar) {
				StretchBlt(DC,0,totalHeight, menuWidth, popupSubHeight, memDC, 0, 0,backBMP.x, backBMP.y, SRCCOPY);
			} else {
				for(int x=0; x<menuWidth; x+=backBMP.x)
					for(int y=totalHeight+1; y<(totalHeight+popupSubHeight); y+=backBMP.y) {
						bWidth = x+backBMP.x<menuWidth?backBMP.x:x<menuWidth?menuWidth-x:menuWidth;
						bHeight = y+backBMP.y<(totalHeight+popupSubHeight)?backBMP.y:y<popupSubHeight?popupSubHeight-y:popupSubHeight;
						BitBlt(DC, x, y, bWidth, bHeight, memDC, 0, 0, SRCCOPY);
					}
			}
		} else {
			hBrush = CreateSolidBrush(normBack);
			SetRect(&r, 0, totalHeight,minWidth, totalHeight+titleHeight);
			FillRect(DC, &r, hBrush);
			DeleteObject(hBrush);
		}
    	SetTextColor(DC,  normColor);
	   	SetBkMode(DC, TRANSPARENT);
    	r.right = backBMP.x-popupTextOffset;
    }
	//Draw Icon
	if (showPopupIcons) {
		if (item->hIcon != NULL)
			DrawIconEx(DC, popupTextOffset, totalHeight+popupTop, item->hIcon, popupIconSize, popupIconSize, 0, NULL, DI_NORMAL); // DI_DEFAULTSIZE
		r.left = popupTextOffset+popupIconSize+4;
	} else
		r.left = popupTextOffset;
   	r.top= totalHeight;
	r.bottom = totalHeight + popupSubHeight;
   	DrawText(DC, item->name, lstrlen(item->name), &r, DT_VCENTER|DT_SINGLELINE|DT_NOCLIP);

   	if (!noPopupBevel) {
		paintPopupBevel(hwnd, item, totalHeight, minWidth);
    }
   	// Paint little folder arrow thingy
   	if (item->SMenu != NULL && popupFolderIcon) {
		paintFolderArrow(hwnd, item, totalHeight, minWidth);
   	}
   	SelectObject(DC, oldFont);
	DeleteObject(hBrush);
   	ReleaseDC(hwnd, DC);
   	EndPaint(hwnd,&ps);	
   	DeleteDC(memDC);
}

Popup *Popup::getPopup(HWND hwnd) {
	Popup *temp;
	BOOL found = FALSE;

	temp = &popup;
	if (temp->hwnd != hwnd) {
		while (temp->child != NULL) {
			temp = temp->child;
			if (temp->hwnd == hwnd) {
				found = TRUE;
				break;
			}
		}
		if (!found) {
			temp = getPopupEx(hwnd, &popup);
		}
	}
	return temp;
}

Popup *Popup::getPopupEx(HWND hwnd, Popup *pop) {
	menuItem *temp;
	Popup *found = NULL;

	temp = pop->root;
	while (temp != NULL) {
		if (temp->SMenu != NULL) {
			if (temp->SMenu->hwnd == hwnd) {
				found = temp->SMenu;
				break;
			} else {
				found = getPopupEx(hwnd, temp->SMenu);
				if (found != NULL)
					break;
			}
		}
		temp = temp->next;
	}
	return found;
}

void Popup::paintPopup() {
    HFONT oldFont;
    RECT r;
    HDC DC, memDC;
    PAINTSTRUCT ps;
	HBRUSH hBrush;
	char *name, temp[MAX_PATH], full[MAX_PATH];
	int menuWidth, bWidth, bHeight;
    
	name = this->root->itemLevel==0?popupMenuName:this->parent->isSelected->name;
    menuItem *iTemp = this->root;
	menuWidth = this->menuWidth;

    DC = BeginPaint(this->hwnd,&ps);
    memDC = CreateCompatibleDC(DC);
    oldFont = (HFONT) SelectObject(DC, popupFont);
	if (!noPopupPix) {
	    //blits out title info
		SelectObject(memDC, titleBMP.bitmap);
		if (shrinkPopupBar) {
			StretchBlt(DC,0,0, menuWidth, titleHeight, memDC, 0, 0,titleBMP.x, titleBMP.y, SRCCOPY);
		} else {
			for(int x=0; x<menuWidth; x+=titleBMP.x)
				for(int y=0; y<titleHeight; y+=titleBMP.y) {
					bWidth = x+titleBMP.x<menuWidth?titleBMP.x:x<menuWidth?menuWidth-x:menuWidth;
					bHeight = y+titleBMP.y<titleHeight?titleBMP.y:y<titleHeight?titleHeight-y:titleHeight;
					BitBlt(DC, x, y, bWidth, bHeight, memDC, 0, 0, SRCCOPY);
				}
		}
	} else {
		hBrush = CreateSolidBrush(titleBack);
		SetRect(&r, 0, 0, menuWidth, titleHeight);
		FillRect(DC, &r, hBrush);
		DeleteObject(hBrush);
	}
    SetBkMode(DC, TRANSPARENT);

    r.left = popupTextOffset;
    r.right = titleBMP.x-popupTextOffset;
    r.top= 0;
    r.bottom = titleHeight;
    SetTextColor(DC,  titleColor);
	DrawText(DC, name, lstrlen(name), &r, DT_VCENTER|DT_SINGLELINE|DT_NOCLIP);
    SelectObject(DC, oldFont);
    ReleaseDC(this->hwnd, DC);

    int totalHeight = titleHeight;
    //blits out each menu item, in order that they where entered into the RC
   	DC = GetDC(this->hwnd);
   	oldFont = (HFONT) SelectObject(DC, popupFont);
   	SelectObject(memDC, backBMP.bitmap);
    while(iTemp != NULL)
    {
		if ((iTemp->command[0] == '\0') && (iTemp->name[0] == '\0')) {
			iTemp = iTemp->next;
			continue;
		}
		if (iTemp->isDynamic && !this->isSubOpen) {
			popupLevel = iTemp->itemLevel + 1;
			if (!strncmp(iTemp->command, "!PopupTasks", 11)) {
				buildTaskList(iTemp);
			} else {
				// The sub menu check is needed to prevent destruction of dynamic menus when a top level menu is redrawn
				delete(iTemp->SMenu);
				strcpy(temp, iTemp->command+20);
				ExpandEnvironmentStrings(temp, full, sizeof(full));
				fillThisDir(full, iTemp);
				if (showPopupIcons)
					GetItemIcon(iTemp);
			}
		}
		if (!noPopupPix) {
			if (shrinkPopupBar) {
				StretchBlt(DC,0,totalHeight, menuWidth, popupSubHeight, memDC, 0, 0,backBMP.x, backBMP.y, SRCCOPY);
			} else {
				for(int x=0; x<menuWidth; x+=backBMP.x)
					for(int y=totalHeight+1; y<(totalHeight+popupSubHeight); y+=backBMP.y) {
						bWidth = x+backBMP.x<menuWidth?backBMP.x:x<menuWidth?menuWidth-x:menuWidth;
						bHeight = y+backBMP.y<(totalHeight+popupSubHeight)?backBMP.y:y<popupSubHeight?popupSubHeight-y:popupSubHeight;
						BitBlt(DC, x, y, bWidth, bHeight, memDC, 0, 0, SRCCOPY);
					}
			}
		} else {
			hBrush = CreateSolidBrush(normBack);
			SetRect(&r, 0, totalHeight, menuWidth, totalHeight+titleHeight);
			FillRect(DC, &r, hBrush);
			DeleteObject(hBrush);
		}
		SetTextColor(DC,  normColor);
    	SetBkMode(DC, TRANSPARENT);
		//Draw Icon
		if (showPopupIcons) {
			if (iTemp->hIcon != NULL)
				DrawIconEx(DC, popupTextOffset, totalHeight+popupTop, iTemp->hIcon, popupIconSize, popupIconSize, 0, NULL, DI_NORMAL); //DI_DEFAULTSIZE
			r.left = popupTextOffset+popupIconSize+4;
		} else
			r.left = popupTextOffset;
    	r.right = backBMP.x-popupTextOffset;
    	r.top= totalHeight;
    	r.bottom = totalHeight + titleHeight;
    	DrawText(DC, iTemp->name, lstrlen(iTemp->name), &r, DT_VCENTER|DT_SINGLELINE|DT_NOCLIP);

    	if (!noPopupBevel)
    	{
			paintPopupBevel(this->hwnd, iTemp, totalHeight, this->menuWidth);
    	}
    	// Paint little folder arrow thingy
    	if (iTemp->SMenu != NULL && popupFolderIcon)
    	{
			paintFolderArrow(this->hwnd, iTemp, totalHeight, this->menuWidth);
    	}
		totalHeight = totalHeight + popupSubHeight;
    	iTemp = iTemp->next;
    }
	if (bottomBMP.bitmap) {
		if (!noPopupPix) {
			SelectObject(memDC, bottomBMP.bitmap);
			if (shrinkPopupBar) {
				StretchBlt(DC,0,totalHeight, menuWidth, popupSubHeight, memDC, 0, 0,bottomBMP.x, bottomBMP.y, SRCCOPY);
			} else {
				for(int x=0; x<menuWidth; x+=bottomBMP.x)
					for(int y=totalHeight+1; y<(totalHeight+popupSubHeight); y+=bottomBMP.y) {
						bWidth = x+bottomBMP.x<menuWidth?bottomBMP.x:x<menuWidth?menuWidth-x:menuWidth;
						bHeight = y+bottomBMP.y<(totalHeight+popupSubHeight)?bottomBMP.y:y<popupSubHeight?popupSubHeight-y:popupSubHeight;
						BitBlt(DC, x, y, bWidth, bHeight, memDC, 0, 0, SRCCOPY);
					}
			}
		} else {
			hBrush = CreateSolidBrush(titleBack);
			SetRect(&r, 0, totalHeight, menuWidth, totalHeight+titleHeight);
			FillRect(DC, &r, hBrush);
			DeleteObject(hBrush);
		}
	}	
	SelectObject(DC, oldFont);
   	ReleaseDC(this->hwnd, DC);
    EndPaint(this->hwnd,&ps);	

    //displays the window
    DeleteDC(memDC);
}

void Popup::closeAll(Popup *pop) {
	menuItem *temp;

	temp = pop->root;
	while (temp != NULL) {
		if (temp->SMenu != NULL) {
			if (temp->SMenu->pinned) {
				temp->SMenu->closeSubMenus();
				temp->SMenu->isSubOpen = FALSE;
				temp->SMenu->isSelected = NULL;
			} else {
				if (temp->SMenu->child != NULL)
					closeAll(temp->SMenu);
				if (temp->SMenu->parent != NULL) {
					if (temp->SMenu->parent->isSubOpen) {
						temp->SMenu->parent->closeSubMenus();
						temp->SMenu->parent->isSubOpen = FALSE;
					}
				}
			}
		}
		temp = temp->next;
	}
	if (pop->parent == NULL && !pop->isSubOpen && !pop->pinned)
		ShowWindow(pop->hwnd, SW_HIDE);
/* _TAG_	sent to LS to have the desktop know the popups are hidden */
	SendMessage(parentWin, LM_POPUPHIDDEN, NULL, NULL);
/* _TAG_ END */
}

menuItem *Popup::keyItem(int c) {
	menuItem *temp;

	if (this->isSelected != NULL) {
		if (this->isSelected->next != NULL) {
			temp = this->isSelected->next;
		} else {
			temp = this->root;
		}
	} else {
		temp = this->root;
	}
	while (temp != NULL) {
		if (temp->name[0] == c) {
			return temp;
		}
		temp = temp->next;
	}
	temp = this->root;
	while (temp != this->isSelected) {
		if (temp->name[0] == c) {
			return temp;
		}
		temp = temp->next;
	}
	return NULL;
}
////////////////////////////////////////////////////////////////////////////////
//END CLASS DEFINITION 
////////////////////////////////////////////////////////////////////////////////

menuItem *GetItem(menuItem *root, int num) {
	menuItem *temp = root;
	while (temp != NULL) {
		if (temp->itemNo == num)
			break;
		temp = temp->next;
	}
	return temp;
}

//cleanup, not much to do
int quitModule(HINSTANCE dllInst)
{
    popup.closeSubMenus();
    DestroyWindow(hMainWnd);
    DeleteObject(popupFont);
	if (titleBMP.bitmap)
		DeleteObject(titleBMP.bitmap);
	if (backBMP.bitmap)
		DeleteObject(backBMP.bitmap);
	if (selectBMP.bitmap)
		DeleteObject(selectBMP.bitmap);
	if (bottomBMP.bitmap)
		DeleteObject(bottomBMP.bitmap);
	if (showPopupIcons) {
		DeleteObject(popupDefaultIcon);
	}
    UnregisterClass(szAppName,dllInst); // unregister window class
    return 1;
}

//now defunct entry point, will take it out if somone complains
int initModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType* wd)
{
    return initModuleEx (ParentWnd, dllInst, wd->lsPath);
}

//DLL entry point
int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
    appInstance = dllInst;
    parentWin = ParentWnd;

    //set all menus to NON-selected mode
    //for(int SE = 0; SE < 255; selected[SE] = FALSE, SE++);
    //parse file and fill popup class structure
    initializePopupMenu();
    
    memset(&wc,0,sizeof(wc));
    wc.lpfnWndProc = (WNDPROC)&Popup::wndProc;// our window procedure
    //wc.hbrBackground = (HBRUSH) GetStockObject(LTGRAY_BRUSH);
    wc.hInstance = appInstance;			
    wc.lpszClassName = szAppName;// our window class name
    wc.style = 0;

    if (!RegisterClass(&wc)) 
    {
    	MessageBox(NULL,"Error registering window class",szAppName,MB_OK);
    	return 1;
    }
    hMainWnd = CreateWindowEx(
        	WS_EX_TOPMOST|WS_EX_TOOLWINDOW,//WS_EX_TRANSPARENT,									// exstyles 
    		szAppName,								// our window class name
    		"",											// use description for a window title
    		WS_POPUP,				
    		0, 0,									// position 
    		popup.menuWidth,(popup.totalHeight + (titleHeight*2)),				// width & height of window
    		ParentWnd,								// parent window 
    		NULL,									// no menu
    		appInstance,								// hInstance of DLL
    		NULL);									// no window creation data

    SetWindowLong(hMainWnd, GWL_USERDATA, magicDWord);
	if (transBlt && !noPopupPix) {
		SetWindowRgn(hMainWnd, popup.region, FALSE);
	}
	popup.hwnd = hMainWnd;

    //create the message id's that popup.dll will use
    int Msgs[10];
    Msgs[0] = LM_POPUP;
    Msgs[1] = LM_HIDEPOPUP;
    Msgs[2] = LM_GETREVID;
	Msgs[3] = 0;
    SendMessage (ParentWnd, LM_REGISTERMESSAGE, (WPARAM)hMainWnd, (LPARAM)Msgs);
	
	for (int i=0; i<256; i++)
		wentRight[i] = FALSE;
    return 0;
}

void paintFolderArrow(HWND hwnd, menuItem *item, int totalHeight, int width)
{
	HDC hDC;
	HPEN hOldPen;

	hDC = GetDC(hwnd);
	// First 2 lines are dark gray
    HPEN hPen = CreatePen(PS_SOLID, 1, RGB(0x40,0x40,0x40));
    hOldPen = (HPEN) SelectObject(hDC, hPen);
    MoveToEx(hDC, width-12, totalHeight+popupSubHeight-5, NULL);
    LineTo(hDC, width-12, totalHeight+4);
    LineTo(hDC, width-5, totalHeight+(popupSubHeight/2));
    SelectObject(hDC, hOldPen);
    DeleteObject(hPen);

    // Finish it off with a white line
    hPen = CreatePen(PS_SOLID, 1, RGB(255,255,255));
    hOldPen = (HPEN) SelectObject(hDC, hPen);
    LineTo(hDC, width-12, totalHeight+popupSubHeight-5);
    SelectObject(hDC, hOldPen);
    DeleteObject(hPen);
	ReleaseDC(hwnd, hDC);
}

void paintPopupBevel(HWND hwnd, menuItem* item, int totalHeight, int width)
{
	HDC hDC;
	HPEN hPen;
	HPEN hOldPen;

	hDC = GetDC(hwnd);
	// First we draw the white frame on the left/top
	hPen = CreatePen(PS_SOLID, 1, RGB(255,255,255));
	hOldPen = (HPEN) SelectObject(hDC, hPen);
	MoveToEx(hDC, 0, totalHeight, NULL);
	LineTo(hDC, width, totalHeight);
	MoveToEx(hDC, 0, totalHeight, NULL);
	LineTo(hDC, 0, totalHeight+popupSubHeight-1);
	SelectObject(hDC, hOldPen);
	DeleteObject(hPen);

	// Now the black frame on the bottom/right
	hPen = CreatePen(PS_SOLID, 1, RGB(0,0,0));
	hOldPen = (HPEN) SelectObject(hDC, hPen);
	MoveToEx(hDC, width-1, totalHeight, NULL);
	LineTo(hDC, width-1, totalHeight+popupSubHeight-1);
	MoveToEx(hDC, 0, totalHeight+popupSubHeight-1, NULL);
	LineTo(hDC, width-1, totalHeight+popupSubHeight-1);
	SelectObject(hDC, hOldPen);
	DeleteObject(hPen);
	ReleaseDC(hwnd, hDC);
}

void executeItem(menuItem* pItem)
{
	if (!stricmp(pItem->command, "t@sk")){
		windowType *winList;
		HWND hwnd;
		char title[MAX_PATH];
		int maxWin;

		winList = (windowType *)SendMessage(parentWin, LM_WINLIST, 0, 0);
		maxWin = (int) SendMessage(parentWin, LM_WINLIST, 1, 0);
		for (int n = 0; n < maxWin && winList[n].Handle; n++) {
			hwnd = winList[n].Handle;
			GetWindowText(hwnd, (char *)&title, MAX_PATH);
			if (!strcmp(pItem->name, (char *)&title)){
				if (IsIconic(hwnd))
					OpenIcon(hwnd);
				SetForegroundWindow(hwnd);
				break;
			}
		}
	} else {
		SHELLEXECUTEINFO si;
		char workDirectory[MAX_PATH];
		char drive[_MAX_DRIVE];
		char dir[_MAX_DIR];
		char fname[_MAX_FNAME]; // not used
		char ext[_MAX_EXT]; // not used

		_splitpath(pItem->command, drive, dir, fname, ext);
		strcpy(workDirectory, drive);
		strcat(workDirectory, dir);

		ZeroMemory(&si, sizeof(si));
		si.cbSize = sizeof(SHELLEXECUTEINFO);

		if(stricmp(ext, ".lnk") == 0)
			// the link files contain their own directory and
			// we should not force any other directory upon it.
			si.lpDirectory = NULL;
		else
			// .exe, .com, .bat, ... often need to be executed from their
			// directory
			si.lpDirectory = workDirectory;
		
		si.lpVerb = NULL;
		si.nShow = 1;
		si.fMask = SEE_MASK_DOENVSUBST;
		si.lpFile = pItem->command;
		si.lpParameters = pItem->params;
		ShellExecuteEx(&si);
	}
}

void initializePopupMenu() 
{
    FILE *f = NULL;
    char buffer[4096];
    char token1[4096], token2[4096], token3[4096], token4[4096], token5[4096], extra_text[4096];
    char* tokens[5];
	char iname[MAX_PATH];
    fileItem * item = popupFile;
    
    GetRCString("HotListName", popupMenuName, "", 256);
    GetRCString("PopupTitlePix", titleBMP.name	, "", 256);
    GetRCString("PopupEntryPix", backBMP.name	, "", 256);
    GetRCString("PopupSelEntryPix", selectBMP.name	, "", 256);
	GetRCString("PopupBottomPix", bottomBMP.name, "", 256);
    GetRCString("PopupFontFace", popupFontFace, "Arial", 256);
	if (strlen(titleBMP.name)==0 || strlen(backBMP.name)==0 || strlen(selectBMP.name)==0) 
		noPopupPix = TRUE;
	popupFolderIcon = GetRCBool("NoPopupFolderIcon", FALSE);
	transBlt = GetRCBool("NoPopupTransparent", FALSE);
    noPopupBevel = GetRCBool("NoPopupBevel", TRUE);
    popupFontHeight = GetRCInt("PopupFontHeight", 16);
    titleColor = GetRCColor("PopupTitleColor", 0xffffff);
    normColor = GetRCColor("PopupEntryColor", 0x000000);
    selColor = GetRCColor("PopupSelEntryColor", 0x000000);
    popupMenuDelay = GetRCInt("PopupMenuDelay",0);
    popupMinWidth = GetRCInt("minpopupwidth", 100);
    popupSubHeight = GetRCInt("PopupSubmenuHeight", 20);
	titleHeight = GetRCInt("PopupSubmenuHeight", 20);
	popupFixedWidth = GetRCInt("PopupFixedWidth", 0);
	isFixedWidth = popupFixedWidth==0?FALSE:TRUE;
	popupTextOffset = GetRCInt("PopupTextOffset", 5);
	showPopupIcons = GetRCBool("ShowPopupIcons", TRUE);
	if (showPopupIcons) {
		popupIconSize = GetRCInt("PopupIconSize", 20);
		GetRCString("PopupDefaultIcon", iname, "default.ico", 256);
		popupDefaultIcon = LoadLSIcon(iname, NULL);
		popupIconSize = popupIconSize>popupSubHeight?popupSubHeight:popupIconSize;
		popupTop = (popupSubHeight-popupIconSize)/2;
	} else 
		popupIconSize = 0;
	// Bottom bitmap is loaded as there is no setting to indicate whether it is drawn or not - yet ...
	bottomBMP.bitmap = LoadLSImage(bottomBMP.name, NULL);
	if (!noPopupPix) {
	    //loads up images i will eventualy seperate this into its own function
		backBMP.bitmap = LoadLSImage(backBMP.name, NULL);
	    titleBMP.bitmap = LoadLSImage(titleBMP.name, NULL);
		selectBMP.bitmap = LoadLSImage(selectBMP.name, NULL); 

	    GetLSBitmapSize(selectBMP.bitmap, &selectBMP.x, &selectBMP.y);
		GetLSBitmapSize(backBMP.bitmap, &backBMP.x, &backBMP.y);
	    GetLSBitmapSize(titleBMP.bitmap, &titleBMP.x, &titleBMP.y);
		GetLSBitmapSize(bottomBMP.bitmap, &bottomBMP.x, &bottomBMP.y);

		//mian's transparency stuff
		selectBMP.region = BitmapToRegion(selectBMP.bitmap, RGB(255,0,255), 0x10101010, 0, 0);
		titleBMP.region = BitmapToRegion(titleBMP.bitmap, RGB(255,0,255), 0x10101010, 0, 0);
		backBMP.region = BitmapToRegion(backBMP.bitmap, RGB(255,0,255), 0x10101010, 0, 0);
		bottomBMP.region = BitmapToRegion(bottomBMP.bitmap, RGB(255,0,255), 0x10101010, 0, 0); 
		//Bitmap shrink/tile option
		shrinkPopupBar = GetRCBool("ShrinkPopupBar", TRUE);
	} else {
	    titleBack = GetRCColor("PopupTitleBack", 0x000000);
            normBack = GetRCColor("PopupNormalBack", 0xa4a0a0);
	    selBack = GetRCColor("PopupSelBack", 0xffffff);
	}

    popupFile = new fileItem;
    item = popupFile;

    tokens[0] = token1;
    tokens[1] = token2;
    tokens[2] = token3;
    tokens[3] = token4;
    tokens[4] = token5;

    buffer[0] = 0;
    f = LCOpen (NULL);
    int keeper = 0;
    
    while (LCReadNextConfig (f, "*popup", buffer, sizeof (buffer)))
    {
     	int count = 0, count2 = 0;
    	char truly_extra_text[4096];
    	if(keeper == 0)
    		keeper = 1;
    	else
    	{
    		item->next = new fileItem;
    		item = item->next;
    	}
    	token1[0] = token2[0] = token3[0] = token4[0] = token5[0] = extra_text[0] = truly_extra_text[0] = '\0';
    	count = LCTokenize (buffer, tokens, 3, extra_text);
    	
    	switch (count)
    	{
    	case 3:											// *Popup
    		strcpy(item->itemName, token2);				// "Name"
    		if (atoi(token3))
    		{
    			item->itemHeight = atoi(token3);			// Height
    			count2 = LCTokenize(extra_text, tokens, 2, truly_extra_text);
    			strcpy(item->bmpName, token1);				// Bitmap.bmp
    			strcpy(item->popupCommand, token2);			// Command.exe
    			strcpy(item->commandParam, truly_extra_text);		// -command_params
    		}
    		else
    		{
    			item->itemHeight = popupSubHeight;
    			strcpy(item->bmpName, backBMP.name);
    			if (!stricmp(token3, "Folder"))
    			{
    				strcpy(item->popupCommand, "!Folder");
    			}
    			else
    			{
    				strcpy(item->popupCommand, token3);
    			}
    			strcpy(item->commandParam, extra_text);
    		}
    		break;

    	case 2:
    		strcpy(item->popupCommand, token2);
    	}
    }		
    LCClose(f);
    //copys the popupFile structure into our popup strucure
    //in later revisions error checking will go here
    treeItem = popupFile;
	buildPopupTree(&popup);

    popupFont = CreateFont(
    		popupFontHeight,
    		0,
    		0,
    		0,
    		FW_NORMAL,
    		FALSE,
    		FALSE,
    		FALSE,
    		DEFAULT_CHARSET,
    		OUT_DEFAULT_PRECIS,
    		CLIP_DEFAULT_PRECIS,
    		DEFAULT_QUALITY,
    		DEFAULT_PITCH,
    		popupFontFace);

	if (showPopupIcons)
		_beginthread(GetItemIcon, 0, popup.root);
}

void buildPopupTree(Popup *pop) {
	int count = 1;

    while(treeItem != NULL) {
    	if(treeItem == NULL || !stricmp(treeItem->popupCommand, "~folder"))
    		break;
    	
    	pop->newMenuItem();
    	strcpy(pop->current->name, treeItem->itemName);
    	strcpy(pop->current->command, treeItem->popupCommand);
    	strcpy(pop->current->bmp, treeItem->bmpName);
    	strcpy(pop->current->params, treeItem->commandParam);
    	pop->current->itemLevel = popupLevel;
		pop->current->itemNo = count;
    	if(!strncmp(pop->current->command, "!PopupFolder", 12))
    	{
			char temp[MAX_PATH], real[MAX_PATH];
			strcpy(temp, pop->current->command+13);
			popupLevel++;
			ExpandEnvironmentStrings(temp, real, sizeof(real));
			fillDirectory(real, pop->current);
			popupLevel--;
			pop->current->hIcon = popupDefaultIcon;
    	}
		else if(!strncmp(pop->current->command, "!PopupDynamicFolder", 19))
		{
			pop->current -> isDynamic = TRUE;
			pop->current->hIcon = popupDefaultIcon;
		}
		else if(!strncmp(pop->current->command, "!PopupTasks", 11))
		{
			pop->current -> isDynamic = TRUE;
			pop->current->hIcon = popupDefaultIcon;
		}
    	else if(!strncmp(pop->current->command, "!Folder", 7))
    	{
			pop->current->hIcon = popupDefaultIcon;
			Popup *sub = new Popup();
			pop->current->SMenu = sub;
			popupLevel++;
			treeItem = treeItem->next;
			buildPopupTree(sub);
			popupLevel--;
		} 
		if (treeItem != NULL)
    		treeItem = treeItem->next;
		count++;
    }
	setPopupValues(pop);
}

void buildTaskList(menuItem *target) {
	windowType *winList;
	char title[MAX_PATH];
	BOOL noList = TRUE;
	int n, maxWin;

	target->SMenu = new Popup;
	winList = (windowType *)SendMessage(parentWin, LM_WINLIST, 0, 0);
	maxWin = (int) SendMessage(parentWin, LM_WINLIST, 1, 0);
	for (n = 0; n < maxWin && winList[n].Handle; n++)
	{
		HICON hIcon = NULL;
		noList = FALSE;

		SendMessageTimeout(winList[n].Handle, WM_GETICON, 1, 0, 0, 250, (unsigned long *)&hIcon);
		if (!hIcon) 
			SendMessageTimeout(winList[n].Handle, WM_QUERYDRAGICON, 0, 0, 0, 1000, (unsigned long *)&hIcon);
		if (!hIcon) 
			hIcon = (HICON)GetClassLong(winList[n].Handle, GCL_HICON);
		if (!hIcon) 
			hIcon = (HICON)GetClassLong(winList[n].Handle, GCL_HICONSM);
	    target->SMenu->newMenuItem();
		title[0] = 0;
		if (winList[n].Handle != NULL) {
			GetWindowText(winList[n].Handle, (char *)&title, MAX_PATH);
		    strcpy(target->SMenu->current->name, title);
			target->SMenu->current->hIcon = hIcon;
			strcpy(target->SMenu->current->command, "T@sk");
			target->SMenu->current->itemLevel = popupLevel;
		//	target->SMenu->current->hwnd = winList[n].Handle;
		}
	}
	if (noList) {
		target->SMenu->newMenuItem();
    	strcpy(target->SMenu->current->name, "No Tasks");
		strcpy(target->SMenu->current->command,"!none");
		target->SMenu->current->itemLevel = popupLevel;
	}
	popupLevel++;
	setPopupValues(target->SMenu);
}

void setPopupValues(Popup *pop) {
    menuItem *counter = pop->root;
	pop->totalHeight = 0;
	int maxWidth = popupMinWidth;
	SIZE name;
	HDC popDC = CreateDC("DISPLAY", NULL, NULL, NULL);
	SelectObject(popDC, popupFont);
	int count = 1;
	while(counter != NULL)
    {
		pop->totalHeight = popupSubHeight + pop->totalHeight;
		GetTextExtentPoint32(popDC,(LPCTSTR)&counter->name,strlen(counter->name),&name);
		if (name.cx+(popupTextOffset*2)+popupIconSize+15 > maxWidth)
			maxWidth = name.cx+(popupTextOffset*2)+popupIconSize+15;
		counter->itemNo = count;
		counter = counter->next;
		count++;
	}
	pop->menuWidth = isFixedWidth?popupFixedWidth:maxWidth;
	if (transBlt && !noPopupPix) {
		pop->region = GetRegion(pop->root);
	}
	// Set close button area for pinning
	pop->close.left = pop->menuWidth - 11;
	pop->close.top = 3;
	pop->close.right = pop->menuWidth - 3;
	pop->close.bottom = 11;
	DeleteDC(popDC);
}

HRGN GetRegion(menuItem *item) {
	menuItem *temp;
	HRGN mainRgn, clipRgn;
	int height = titleHeight;

	mainRgn = CreateRectRgn(0, 0, 0, 0);
	CombineRgn(mainRgn, mainRgn, titleBMP.region, RGN_OR);
	temp = item;
	while (temp != NULL) {
		clipRgn = CreateRectRgn(0, 0, 0, 0);
		CombineRgn(clipRgn, backBMP.region, NULL, RGN_COPY);
		OffsetRgn(clipRgn, 0, height);
		CombineRgn(mainRgn, mainRgn, clipRgn, RGN_OR);
		DeleteObject(clipRgn);
		height = height + popupSubHeight;
		temp = temp->next;
	}
	if (bottomBMP.bitmap) {
		clipRgn = CreateRectRgn(0, 0, 0, 0);
		CombineRgn(clipRgn, bottomBMP.region, NULL, RGN_COPY);
		OffsetRgn(clipRgn, 0, height);
		CombineRgn(mainRgn, mainRgn, clipRgn, RGN_OR);
		DeleteObject(clipRgn);
	}
	return mainRgn;
}

void displayPopup(HWND hwnd, int x, int y)
{
	int height;
	height = bottomBMP.bitmap?titleHeight*2:titleHeight;
	popup.onLeft = FALSE;
    
    //Get popup coordinates
	if (x + popup.menuWidth > GetSystemMetrics(SM_CXSCREEN)) {
		x -= popup.menuWidth;
		popup.onLeft = TRUE;
	}

	if ((y + popup.totalHeight + height) > GetSystemMetrics(SM_CYSCREEN))
		y = GetSystemMetrics(SM_CYSCREEN) - (popup.totalHeight + height);
	SetWindowPos(hwnd, HWND_TOP, x, y, popup.menuWidth,(popup.totalHeight + height), 
    	SWP_SHOWWINDOW);
    popup.paintPopup();
}

int fillDirectorySortFunc(const void *a, const void *b)
{
    WIN32_FIND_DATA *f1 = (WIN32_FIND_DATA *)a;
    WIN32_FIND_DATA *f2 = (WIN32_FIND_DATA *)b;

    if (f1->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
    {
    	if (f2->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
    		return stricmp(f1->cFileName, f2->cFileName);
    	else
    		return -1;
    }
    if (f2->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
    	return 1;

    return stricmp(f1->cFileName, f2->cFileName);
}

//parses a directory and fills the popup data structure with the results
int fillDirectory(char *dirName, menuItem *target)
{
    int l=0;	
    int j, k, still=1;

    WIN32_FIND_DATA fd;
    HANDLE h;
    char temp2[MAX_PATH];
    strcpy(temp2, dirName);
    strcat(temp2, "\\*");
    h = FindFirstFile(temp2, &fd);
    if (h == INVALID_HANDLE_VALUE)
    	return 1;
    
    target->SMenu = new Popup;
    menuItem *walker = target;
    WIN32_FIND_DATA fileTable[255];
    
    for (j=0;j<255 && still;j++)
    {
    	fileTable[j] = fd;
    	still = FindNextFile(h, &fd);
    }

    qsort((void *)fileTable, j, sizeof(WIN32_FIND_DATA), fillDirectorySortFunc);

    for (k=j,j=0;l<255 && j<k;l++,j++)
    {
    	if (fileTable[j].dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
        {
    		if (!strcmp(fileTable[j].cFileName, ".") || !strcmp(fileTable[j].cFileName, ".."))
           	{
            	l--;
    		}
    		else
    		{
    			char t[MAX_PATH];
    			walker->SMenu->newMenuItem();// = new menuItem;
    			strcpy(walker->SMenu->current->name, fileTable[j].cFileName);
    			strcpy(t, dirName);
    			strcat(t, "\\");
    			strcat(t, fileTable[j].cFileName);
    			walker->SMenu->current->itemLevel = popupLevel;
    			popupLevel++;
    			int temp2 = fillDirectory(t, walker->SMenu->current);
    			if(temp2)
    			{
    				delete walker->SMenu->current->SMenu;
    				walker->SMenu->current->SMenu = NULL;
    				strcpy(walker->SMenu->current->command, t);
    			}
    			popupLevel--;
    		}
    	}
    	else
    	{
    		char temp[5];
    		
    		//strcpy(temp2, "\"");
    		strcpy(temp2, "");
    	 	strcat(temp2, dirName);
    		strcat(temp2, "\\");
    		strcat(temp2, fileTable[j].cFileName);
    		//strcat(temp2, "\"");
       		
    		walker->SMenu->newMenuItem();			
    		strcpy(walker->SMenu->current->command,temp2);
    		strcpy(temp, fileTable[j].cFileName +(strlen(fileTable[j].cFileName) -4));
    		//stripping unwanted extentions from the name
			if(!stricmp(temp, ".lnk") || !stricmp(temp, ".exe") || !stricmp(temp, ".pif") || !stricmp(temp, ".com") || !stricmp(temp, ".rnk") || !stricmp(temp, ".bat")) 
    		{
    			strncpy(walker->SMenu->current->name, fileTable[j].cFileName, 
    				strlen(fileTable[j].cFileName) -4);
    			walker->SMenu->current->name[strlen(fileTable[j].cFileName) -4] = '\0';
    		}
    		else 
				strcpy(walker->SMenu->current->name, fileTable[j].cFileName);
    		walker->SMenu->current->itemLevel = popupLevel;
		}
    }
    FindClose(h);
    if(target->SMenu->root == NULL)
    	return 1;
	else {
		setPopupValues(target->SMenu);
	    return 0;
	}
}
    
//parses a given directory and fills the popup data structure with the results for the current level
void fillThisDir(char *dirName, menuItem *target)
{
    int l=0;	
    int j, k, still=1;
	WIN32_FIND_DATA *fileTable=NULL;

    WIN32_FIND_DATA fd;
    HANDLE h;
    char temp2[MAX_PATH];
    strcpy(temp2, dirName);
    strcat(temp2, "\\*");
    h = FindFirstFile(temp2, &fd);
    if (h == INVALID_HANDLE_VALUE) {
    } else {
	    target->SMenu = new Popup;
		menuItem *walker = target;
    
	    for (j=0;still;j++)
		{
			if (!fileTable)
				fileTable = (WIN32_FIND_DATA *)malloc(sizeof(WIN32_FIND_DATA));
			else
				fileTable = (WIN32_FIND_DATA *)realloc(fileTable, (j+1)*sizeof(WIN32_FIND_DATA));
    		fileTable[j] = fd;
	    	still = FindNextFile(h, &fd);
		}
	    qsort((void *)fileTable, j, sizeof(WIN32_FIND_DATA), fillDirectorySortFunc);

		for (k=j,j=0;j<k;j++)
		{
	    	if (fileTable[j].dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		    {
    			if (!strcmp(fileTable[j].cFileName, ".") || !strcmp(fileTable[j].cFileName, ".."))
           		{
				} else {
					char t[MAX_PATH];
    				walker->SMenu->newMenuItem();// = new menuItem;
    				strcpy(walker->SMenu->current->name, fileTable[j].cFileName);
					strcpy(t, "!PopupDynamicFolder:");
    				strcat(t, dirName);
	    			strcat(t, "\\");
					strcat(t, fileTable[j].cFileName);
					strcpy(walker->SMenu->current->command, t);
    				walker->SMenu->current->itemLevel = popupLevel;
					walker->SMenu->current->isDynamic = TRUE;
	    		}
			} else {
	    		char temp[5];
	    		strcpy(temp2, "");
			 	strcat(temp2, dirName);
    			strcat(temp2, "\\");
    			strcat(temp2, fileTable[j].cFileName);
       		
	    		walker->SMenu->newMenuItem();			
				strcpy(walker->SMenu->current->command,temp2);
    			strcpy(temp, fileTable[j].cFileName +(strlen(fileTable[j].cFileName) -4));
	    		//stripping unwanted extentions from the name
				if(!stricmp(temp, ".lnk") || !stricmp(temp, ".exe") || !stricmp(temp, ".pif") || !stricmp(temp, ".com") || !stricmp(temp, ".rnk") || !stricmp(temp, ".bat")) 
    			{
	    			strncpy(walker->SMenu->current->name, fileTable[j].cFileName, strlen(fileTable[j].cFileName) -4);
	    			walker->SMenu->current->name[strlen(fileTable[j].cFileName) -4] = '\0';
				} else  
					strcpy(walker->SMenu->current->name, fileTable[j].cFileName);
	    		walker->SMenu->current->itemLevel = popupLevel;
			}
	    }
		FindClose(h);
	}
	if (target->SMenu->root == NULL) {
		target->SMenu->newMenuItem();
    	strcpy(target->SMenu->current->name, "Empty");
		strcpy(target->SMenu->current->command,"!none");
		target->SMenu->current->itemLevel = popupLevel;
	}
	setPopupValues(target->SMenu);
}

void keyedSubMenu(HWND menu, menuItem *showItem) {
	RECT lpMenu;
	int height;

	height=showItem->itemNo*popupSubHeight;
	GetWindowRect(menu, &lpMenu);
	SetCursorPos(lpMenu.left+2, lpMenu.top+height+2);
}

void GetItemIcon(void *pitem) {
	HICON hIcon;
	char *file;
	char icon[MAX_PATH];
	menuItem *item = (menuItem *) pitem;
	char dir[MAX_PATH];
	UINT dsize = MAX_PATH;

	if (!showPopupIcons)
		_endthread();
    while(item != NULL) {
		strcpy(icon, "");
		if (item->SMenu != NULL) {
			item->hIcon = popupDefaultIcon;
			GetItemIcon(item->SMenu->root);
		} else {
			file = strlwr(strdup(item->command));
			if (strstr(file, "explorer")) {
				file = strdup(item->params);
				if (strstr(file, "{21EC2020-3AEA-1069-A2DD-08002B30309D}")) {
					//Control Panel
					strcpy(file, ""); //Default icon
					RegistryLookup("CLSID\\{21EC2020-3AEA-1069-A2DD-08002B30309D}\\DefaultIcon", file);
				} else if (strstr(file, "{208D2C60-3AEA-1069-A2D7-08002B30309D}")) {
					//Network Neigbourhood
					strcpy(file, ""); //Default icon
					RegistryLookup("CLSID\\{208D2C60-3AEA-1069-A2D7-08002B30309D}\\DefaultIcon", file);
				} else if (strstr(file, "{992CFFA0-F557-101A-88EC-00DD010CCC48}")) {
					//Dial-up Networking
					strcpy(file, ""); //Default icon
					RegistryLookup("CLSID\\{992CFFA0-F557-101A-88EC-00DD010CCC48}\\DefaultIcon", file);
				} else if (strstr(file, "{20D04FE0-3AEA-1069-A2D8-08002B30309D}")) {
					//My computer
					strcpy(file, ""); //Default icon
					RegistryLookup("CLSID\\{20D04FE0-3AEA-1069-A2D8-08002B30309D}\\DefaultIcon", file);
				}
				if (strlen(file) != 0) {
					strcpy(icon, ".extract=");
					if (!strstr(file, "\\")) {
						if (strstr(file, "explorer"))
							GetWindowsDirectory(dir, dsize);
						else
							GetSystemDirectory(dir, dsize);
						strcat(icon, dir);
							strcat(icon, "\\");
					}
					strcat(icon, file);
					hIcon = LoadLSIcon(icon, NULL);
					item->hIcon = hIcon;
				}
			} else {
				SHFILEINFO shf;

				SHGetFileInfo(item->command, NULL, &shf, sizeof(shf), SHGFI_ICON | SHGFI_SMALLICON);
				if (shf.hIcon != NULL)
					item->hIcon = shf.hIcon;
				else {
					_searchenv(item->command, "PATH", dir);
					if (strlen(dir) != 0) {
						SHGetFileInfo(dir, NULL, &shf, sizeof(shf), SHGFI_ICON | SHGFI_SMALLICON);
						item->hIcon = shf.hIcon;
					}
				}
			}
		}
		item = item->next;
    }
}

void RegistryLookup(char* subKey, char *value) {
	char regData[256];
	int regSize = 256;
	LONG res;
	HKEY hOpen;

	RegOpenKeyEx(HKEY_CLASSES_ROOT, subKey, 0, KEY_READ, &hOpen);
	res = RegQueryValueEx(hOpen, value, NULL, NULL, (LPBYTE)regData, (LPDWORD)&regSize);
	if (res == ERROR_SUCCESS)
		strcpy(value, regData);
	else
		strcpy(value, "");
	RegCloseKey(hOpen);
}
/*
	$Log: popup.cpp,v $
	Revision 1.1  1999/01/20 22:51:26  cyberian
	
	The old popups implemented with window classes with pinning, transparency etc.
	
*/
